/*
  _===_ _           _                  ______ _     _             _ _ _
 |  ___| |         | |                 | ___ (_)   | |           (_) | |
 | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
 |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
 | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
 \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

 Horloge simple sur l'affichage de la plateforme Z80 PRO-83.
 Code source développé et compilé sur SDCC
 Électro-Bidouilleur, V1, Juillet 2022. https://www.youtube.com/Électro-Bidouilleur
 Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr  
*/

// COMMANDE UTILISÉE AVEC SDCC
// sdcc -mz80 --code-loc 0x0000 --data-loc 0x1000 --stack-loc 0x17FF --no-std-crt0 --out-fmt-ihx /media/4TB_BACKUP/ZAZDISK_D/Proj/Bidouilleur/PRO-83_Plateforme_Z80/Programmes/SDCC/Horloge.c

// REGISTRES SPÉCIAUX
__sfr __at 0x48 chiff_sel;                               // Définition des registres spéciaux de l'affichage
__sfr __at 0x4C segm_sel;                                //           "
__sfr __at 0x40 pio_a_d;                                 // Définition des registres spéciaux du PIO
__sfr __at 0x42 pio_a_c;                                 //       "


// VARIABLES GLOBALES (adressage absolu requis)
__at 0x1000 volatile unsigned char cnt;                  // Compteur de pause
__at 0x1001 volatile unsigned char chif_affich;          // Sélecteur de position du chiffre affiché
__at 0x1002 volatile unsigned char niv_haut_prec;        // Statut du niveau haut/bas de l'horloge de 1 Hz       
__at 0x1003 volatile unsigned char heure[6];             // Liste des 6 chiffre de l'heure affichée


// CONSTANTES (placées en mémoire ROM)
const unsigned char chif_defs[10] = { 0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F , 0x6F };  // Définition des segments d'affichage LED pour chiffres (0 à 9)    


// FONCTION PRINCIPALE
void main(void) 
{   
   pio_a_c = 0x4F;                                       // Configurer le port PIO-A en mode entree (mode 0)
   niv_haut_prec = 0;                                    // Initialiser le statut du niveau haut/bas de l'horloge de 1 Hz

   // Initialisation de l'heure de départ dans les 6 chiffres
   heure[5] = 2;                                         // Dizaine des heure                         
   heure[4] = 3;                                         // Unité des heure                         
   heure[3] = 5;                                         // Dizaine des minutes                   
   heure[2] = 9;                                         // Unité des minutes                     
   heure[1] = 0;                                         // Dizaine des secondes                        
   heure[0] = 0;                                         // Unité des secondes                           
    
   while(1)                                              // Boucle principale
   {      
      if ((pio_a_d & 1) && (!niv_haut_prec))             // Niveau haut sur l'hologe détecté pour la première fois dans la seconde?
      {                                                  // Oui
         niv_haut_prec = 1;                              // Lever le fanion du niveau haut détecté
         heure[0]++;                                     // Incrémenter le chiffre d'unités des secondes
         if (heure[0] == 10)                             // Besoin de débordement dans les dizaines de secondes
         {
            heure[0] = 0;
            heure[1]++;                                  // Incrémenter le chiffre de dizaines des secondes
            if (heure[1] == 6)                           // Besoin de débordement dans les unités de minutes
            {
               heure[1] = 0;
               heure[2]++;                               // Incrémenter le chiffre d'unités des minutes
               if (heure[2] == 10)                       // Besoin de débordement dans les dizaines de minutes
               { 
                  heure[2] = 0;
                  heure[3]++;                            // Incrémenter le chiffre de dizaines des minutes
                  if (heure[3] == 6)                     // Besoin de débordement dans les unités d'heures
                  {
                     heure[3] = 0;
                     heure[4]++;                         // Incrémenter le chiffre d'unités des heures
                     if (heure[4] == 10)                 // Besoin de débordement dans les dizaines d'heures
                     {
                        heure[4] = 0;
                        heure[5]++;                      // Incrémenter le chiffre de dizaines des heures
                     }
                     if ((heure[5] == 2) && (heure[4] == 4))   // 24ème heure atteinte?
                     {                                   // Oui
                        heure[5] = 0;                    // Remettre les heures à 00
                        heure[4] = 0;                    //          "
                     }
                  }
               }
            }
         }
      }           
      else if (!pio_a_d & 1) niv_haut_prec = 0;          // Niveau bas sur l'hologe détecté? Si oui, abaisser le fanion du niveau haut détecté
      
      for (chif_affich = 0; chif_affich < 6; chif_affich++)    // Boucle d'affichage des 6 chiffres de l'heure
      {
         segm_sel = chif_defs[heure[chif_affich]];       // Selectionner les segments de LED associés au chiffre à afficher
         chiff_sel = ~(0b00000001<<chif_affich);         // Sélectionner le chiffre à afficher (logique actif bas)
         pause();                                        // Ajouter une pause pour les yeux
      }
   }
}

// FONCTION GÉNÉRANT UNE PAUSE
void pause(void)
{  
   for (cnt = 0; cnt < 30; cnt++);                       // Boucle de délai
}

// FIN
